<?php

namespace Drupal\access_filter\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Drupal\access_filter\Entity\Filter;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides filter overview form.
 */
class Overview extends FormBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'access_filter_overview';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    if (Settings::get('access_filter_disabled')) {
      drupal_set_message($this->t('Access control is currently disabled in settings.php.'), 'warning');
    }

    $form['filters'] = [
      '#type' => 'table',
      '#attributes' => ['id' => 'filters'],
      '#header' => [
        $this->t('ID'),
        $this->t('Name'),
        $this->t('Conditions'),
        $this->t('Rules'),
        $this->t('Response'),
        $this->t('Weight'),
        $this->t('Operations'),
      ],
      '#empty' => $this->t('No filters available.'),
    ];

    $destination = $this->getDestinationArray();

    $filters = Filter::loadMultiple();
    $form_state->set('filters', $filters);

    foreach ($filters as $filter) {
      $filter->parse();

      $classes = ['draggable'];
      if (!$filter->status()) {
        $classes[] = 'disabled';
      }

      $row = [
        '#attributes' => ['class' => $classes],
      ];

      $row['id'] = [
        '#markup' => $filter->id,
      ];
      $row['name'] = [
        '#markup' => Html::escape($filter->name),
      ];

      $conditions = [];
      foreach ($filter->parsedConditions as $condition) {
        $conditions[] = $this->t('@type: @summary', [
          '@type' => $condition->getPluginId(),
          '@summary' => $condition->summary(),
        ]);
      }
      $row['conditions'] = [
        '#theme' => 'item_list',
        '#items' => $conditions,
      ];

      $rules = [];
      foreach ($filter->parsedRules as $rule) {
        $rules[] = $this->t('@type: @summary', [
          '@type' => $rule->getPluginId(),
          '@summary' => $rule->summary(),
        ]);
      }
      $row['rules'] = [
        '#theme' => 'item_list',
        '#items' => $rules,
      ];

      $row['response'] = [
        '#markup' => Html::escape($filter->parsedResponse['code']),
      ];

      $operations = [];
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'query' => $destination,
        'url' => Url::fromRoute('entity.access_filter.edit_form', ['access_filter' => $filter->id]),
      ];
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'query' => $destination,
        'url' => Url::fromRoute('entity.access_filter.delete_form', ['access_filter' => $filter->id]),
      ];

      $row['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for filter.'),
        '#title_display' => 'invisible',
        '#delta' => 20,
        '#attributes' => [
          'class' => ['filter-weight'],
        ],
        '#default_value' => $filter->weight,
      ];

      $row['operations'] = [
        '#type' => 'operations',
        '#links' => $operations,
      ];

      $form['filters'][$filter->id] = $row;
    }

    $form['filters']['#tabledrag'][] = [
      'action' => 'order',
      'relationship' => 'sibling',
      'group' => 'filter-weight',
    ];

    $form['actions']['save'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    $form['#attached']['library'][] = 'access_filter/common';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /* @var Filter[] $filters */
    $filters = $form_state->get('filters');
    /* @var Filter[] $changed_filters */
    $changed_filters = [];
    $values = $form_state->getValue('filters');
    foreach ($values as $fid => $value) {
      if ($filters[$fid]->weight != $value) {
        $filters[$fid]->weight = $value['weight'];
        $changed_filters[] = $filters[$fid];
      }
    }

    foreach ($changed_filters as $filter) {
      $filter->save();
    }
  }

}
